interface Nested<Ret = unknown, Args extends any[] = any[]> {
    (key: string, ...args: Args): Ret;
    has(key: string): boolean;
    map: Map<string, Ret>;
}
declare type ExtractArgs<T extends object> = T[keyof T] extends (...args: infer A) => any ? A : any[];
interface DurableState<T extends object, R = unknown, Args extends unknown[] = ExtractArgs<T>> {
    <K = keyof T>(key?: K, ...args: Args): R;
    (target?: T, ext?: T[keyof T], ...args: Args): R;
}
declare type MutableState<T extends object, Args extends unknown[] = ExtractArgs<T>> = T & {
    <K extends keyof T>(key?: K, fun?: T[K]): MutableState<T, Args>;
    (target?: T, ext?: any): MutableState<T, Args>;
};

declare type EventState<T extends object, Args extends unknown[] = ExtractArgs<T>> = T & {
    (key?: keyof T, fun?: T[keyof T]): EventState<T, Args>;
    (target?: Partial<T>): EventState<T, Args>;
};
declare type NestedFun<Ret, Args extends any[] = any[]> = Fun<[
    key: string,
    ...args: Args
], Ret>;
declare type DurableFun<T extends object, Args extends unknown[] = ExtractArgs<T> & any> = Fun<[key: string, ...args: Args]>;
declare type EventFun<T extends object, Args extends unknown[] = ExtractArgs<T>> = Fun<[...args: Args], EventState<T>>;
declare type Fun<Args extends unknown[] = unknown[], Ret extends unknown = unknown> = (...args: Args) => Ret;
declare type OverloadedArgs<T extends (...args: any[]) => unknown> = T extends {
    (...args: infer A1): any;
    (...args: infer A2): any;
} ? A1 | A2 : Parameters<T>;
declare type DurableArgs<T extends object> = OverloadedArgs<DurableState<T>>;
declare type MutableArgs<T extends object> = OverloadedArgs<MutableState<T>>;
declare type EventArgs<T extends object> = OverloadedArgs<DurableState<{
    [key in keyof T]?: [EventFun<T>];
}>>;
declare type RefEvent<T extends object, Target = unknown> = EventState<T & {
    ref: (target: Target) => void;
    mount: (target: Target) => void;
    clean: (target: Target) => void;
    target: Target;
}>;

declare function nested<T, Args extends any[] = any[]>(init: NestedFun<T, Args>): Nested<T, Args>;
declare const durable: <T extends object, Ret = unknown>(fun: DurableFun<T, any>, ret?: Ret) => DurableState<T, Ret extends undefined ? (arg: T, ...args: unknown[]) => Ret | any : Ret, ExtractArgs<T>>;
declare const mutable: <T extends object>(...args: [] | OverloadedArgs<MutableState<T, ExtractArgs<T>>>) => MutableState<T, ExtractArgs<T>>;
declare function event<T extends object>(...args: EventArgs<T> | []): EventState<T, ExtractArgs<T>>;

export { DurableArgs, DurableFun, DurableState, EventArgs, EventFun, EventState, ExtractArgs, Fun, MutableArgs, MutableState, Nested, NestedFun, OverloadedArgs, RefEvent, event as default, durable, event, mutable, nested };
